#!/usr/bin/ruby
$:.unshift File.join(File.dirname(__FILE__))
require 'podnovasync_conf.rb'
require 'net/http'
require 'rexml/document'

begin
require 'logger'
require 'Korundum'

rescue LoadError
    error="Korundum is required for this script, please install it."
    `kdialog --sorry '#{error}'`
    exit
end

class PodnovaSync
    SYNC_INTERVAL = 60
    def initialize
        trap( "SIGTERM" ) {
            cleanup()
        }
        trap("SIGINT"){
            cleanup()
        }
        @log = Logger.new("./podnova_sync.log")
        KDE::CmdLineArgs.init(ARGV, KDE::AboutData.new("chwallpaper", "Change wallpaper", "0.1"))
        @kde_application = KDE::Application.new()
        @dcopclient = KDE::DCOPClient.new
        @dcopclient.attach
        @dcopclient.registerAs("PodnovaSync",false)
        @amarok_playlist_browser = KDE::DCOPRef.new('amarok', 'playlistbrowser')
        @amarok_playlist_browser.setDCOPClient(@dcopclient)
        @config_done = false

    end
    
    def cleanup()
            @log.debug( "Recieved SIGTERM, cleanup and close." )
            exit
    end
    
    def do_configuration
        if !@config_done
            @config_done = true
            config = PodnovaOpmlConfig.new()
            @kde_application.setMainWidget(config)
            config.show()
            @kde_application.exec()
        else
            error="Config has already run, please restart the script to run it again."
            `kdialog --sorry '#{error}'`
        end
    end

    def sync_podcasts
        opml_url = get_config_value("opmlUrl")
        if !opml_url.nil?
            opml = Net::HTTP.get_response(URI.parse(opml_url)).body
            rss_xml_urls = get_rss_xml_urls(opml)
            rss_xml_urls.each do |url|
                @amarok_playlist_browser.addPodcast(url)
            end
            @amarok_playlist_browser.scanPodcasts()
        end
    end

    def get_rss_xml_urls(opml_string)
        url_list = []
        doc = REXML::Document.new opml_string
        REXML::XPath.each( doc, "/opml/body/outline/outline/@xmlUrl") { |url| url_list << url.to_s }
        return url_list
    end

    def get_config_value(key)
        begin
            File.open("./synpodnova.conf", "r").each_line do |line|
                if line[key]
                    return line.split("=")[1].chomp
                end
            end
        rescue Exception
        end
        return nil
    end
    def run
        sync_podcasts
        @log.debug("Running...")
        syncThread = Thread.new(){
            loop do
                sync_podcasts
                sleep(SYNC_INTERVAL * 60)
            end
        }
        
        loop do
            message = gets().chomp() #Read message from stdin
            if(message == "configure")
                do_configuration()
            end
        end
        syncThread.join()
        @log.debug("Done.")
    end
end

PodnovaSync.new().run
